@@ -17,12 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "xml-entry.h"
-#include <gconf/gconf-internals.h>
+#include "gconf/gconf-internals.h"
 #include <stdlib.h>
 #include <string.h>
 #include <libxml/entities.h>
-#include <libxml/xmlmemory.h>
+#include <libxml/globals.h>
 
 static void
 entry_sync_if_needed(Entry* e);
@@ -64,14 +65,12 @@ entry_new (const gchar* relative_name)
 void
 entry_destroy (Entry* e)
 {
-  if (e->name)
-    g_free(e->name);
+  g_free(e->name);
 
   if (e->cached_value)
     gconf_value_free(e->cached_value);
 
-  if (e->mod_user)
-    g_free(e->mod_user);
+  g_free(e->mod_user);
 
   if (e->node != NULL)
     {
@@ -121,6 +120,10 @@ entry_get_value(Entry* e, const gchar** locales, GError** err)
 
   sl = gconf_schema_get_locale(gconf_value_get_schema(e->cached_value));
 
+  gconf_log (GCL_DEBUG, "Cached schema value has locale \"%s\", looking for %s",
+             sl ? sl : "null",
+             locales && locales [0] ? locales[0] : "null");
+  
   /* optimize most common cases first */
   if (sl == NULL && (locales == NULL ||
                      *locales == NULL))
@@ -163,7 +166,7 @@ void
 entry_set_value(Entry* e, const GConfValue* value)
 {
   g_return_if_fail(e != NULL);
-
+  
   entry_sync_if_needed(e);
   
   if (e->cached_value)
@@ -198,7 +201,7 @@ entry_unset_value     (Entry        *e,
 
           if (error != NULL)
             {
-              gconf_log(GCL_WARNING, _("%s"), error->message);
+              gconf_log(GCL_WARNING, "%s", error->message);
               g_error_free(error);
               error = NULL;
             }
@@ -255,10 +258,9 @@ void
 entry_set_schema_name (Entry        *e,
                        const gchar  *name)
 {
-  if (e->schema_name)
-    g_free(e->schema_name);
+  g_free(e->schema_name);
 
-  e->schema_name = name ? g_strdup(name) : NULL;
+  e->schema_name = g_strdup(name);
   
   e->dirty = TRUE;
 }
@@ -279,8 +281,7 @@ entry_set_mod_user (Entry *e,
 {
   g_return_if_fail(e != NULL);
   
-  if (e->mod_user)
-    g_free(e->mod_user);
+  g_free(e->mod_user);
   e->mod_user = g_strdup(user);
 
   e->dirty = TRUE;
@@ -368,11 +369,13 @@ entry_fill_from_node(Entry* e)
     }
   else if (error != NULL)
     {
-      /* FIXME for nodes with no value stored, but containing a schema name,
-       * we improperly log an error here
+      /* Ignore errors from node_extract_value() if we got a schema name,
+       * since the node's only purpose may be to store the schema name.
        */
-      gconf_log(GCL_WARNING, _("Ignoring XML node `%s', except for possible schema name: %s"),
-                e->name, error->message);
+      if (e->schema_name == NULL)
+        gconf_log (GCL_WARNING,
+                   _("Ignoring XML node `%s': %s"),
+                   e->name, error->message);
       g_error_free(error);
     }
 }
@@ -439,24 +442,24 @@ node_set_schema_value(xmlNodePtr node,
   const gchar* type;
   xmlNodePtr found = NULL;
 
-  sc = gconf_value_get_schema(value);
+  sc = gconf_value_get_schema (value);
 
   /* Set the types */
-  if (sc->list_type != GCONF_VALUE_INVALID)
+  if (gconf_schema_get_list_type (sc) != GCONF_VALUE_INVALID)
     {
-      type = gconf_value_type_to_string(sc->list_type);
+      type = gconf_value_type_to_string(gconf_schema_get_list_type (sc));
       g_assert(type != NULL);
       my_xmlSetProp(node, "list_type", type);
     }
-  if (sc->car_type != GCONF_VALUE_INVALID)
+  if (gconf_schema_get_car_type (sc) != GCONF_VALUE_INVALID)
     {
-      type = gconf_value_type_to_string(sc->car_type);
+      type = gconf_value_type_to_string(gconf_schema_get_car_type (sc));
       g_assert(type != NULL);
       my_xmlSetProp(node, "car_type", type);
     }
-  if (sc->cdr_type != GCONF_VALUE_INVALID)
+  if (gconf_schema_get_cdr_type (sc) != GCONF_VALUE_INVALID)
     {
-      type = gconf_value_type_to_string(sc->cdr_type);
+      type = gconf_value_type_to_string(gconf_schema_get_cdr_type (sc));
       g_assert(type != NULL);
       my_xmlSetProp(node, "cdr_type", type);
     }
@@ -465,8 +468,8 @@ node_set_schema_value(xmlNodePtr node,
   my_xmlSetProp(node, "value", NULL);
 
   /* set the cross-locale attributes */
-  my_xmlSetProp(node, "stype", gconf_value_type_to_string(sc->type));
-  my_xmlSetProp(node, "owner", sc->owner);
+  my_xmlSetProp(node, "stype", gconf_value_type_to_string(gconf_schema_get_type (sc)));
+  my_xmlSetProp(node, "owner", gconf_schema_get_owner (sc));
 
   locale = gconf_schema_get_locale(sc);
 
@@ -478,26 +481,27 @@ node_set_schema_value(xmlNodePtr node,
   found = find_schema_subnode_by_locale(node, locale);
   
   if (found == NULL)
-    found = xmlNewChild(node, NULL, "local_schema", NULL);
+    found = xmlNewChild(node, NULL, (xmlChar *)"local_schema", NULL);
   
   /* OK if these are set to NULL, since that unsets the property */
-  my_xmlSetProp(found, "locale", sc->locale);
-  my_xmlSetProp(found, "short_desc", sc->short_desc);
+  my_xmlSetProp(found, "locale", gconf_schema_get_locale (sc));
+  my_xmlSetProp(found, "short_desc", gconf_schema_get_short_desc (sc));
 
   free_childs(found);
   
-  if (sc->default_value != NULL)
+  if (gconf_schema_get_default_value (sc) != NULL)
     {
       xmlNodePtr default_value_node;
-      default_value_node = xmlNewChild(found, NULL, "default", NULL);
-      node_set_value(default_value_node, sc->default_value);
+      default_value_node = xmlNewChild(found, NULL, (xmlChar *)"default", NULL);
+      node_set_value(default_value_node, gconf_schema_get_default_value (sc));
     }
   
-  if (sc->long_desc)
+  if (gconf_schema_get_long_desc (sc))
     {
       xmlNodePtr ld_node;
       
-      ld_node = xmlNewChild(found, NULL, "longdesc", sc->long_desc);
+      ld_node = xmlNewChild(found, NULL, (xmlChar *)"longdesc", 
+                            (xmlChar *)gconf_schema_get_long_desc (sc));
     }
 }
 
@@ -533,14 +537,14 @@ node_set_value(xmlNodePtr node, GConfValue* value)
     case GCONF_VALUE_STRING:
       {
         xmlNodePtr child;
-        gchar* encoded;
+        xmlChar* encoded;
         
         free_childs(node);
 
         encoded = xmlEncodeEntitiesReentrant(node->doc,
-                                             gconf_value_get_string(value));
+                                             (xmlChar *)gconf_value_get_string(value));
         
-        child = xmlNewChild(node, NULL, "stringvalue",
+        child = xmlNewChild(node, NULL, (xmlChar *)"stringvalue",
                             encoded);
         xmlFree(encoded);
       }
@@ -566,7 +570,7 @@ node_set_value(xmlNodePtr node, GConfValue* value)
           {
             xmlNodePtr child;
             /* this is O(1) because libxml saves the list tail */
-            child = xmlNewChild(node, NULL, "li", NULL);
+            child = xmlNewChild(node, NULL, (xmlChar *)"li", NULL);
 
             g_return_if_fail(list->data != NULL);
             
@@ -583,8 +587,8 @@ node_set_value(xmlNodePtr node, GConfValue* value)
 
         free_childs(node);
 
-        car = xmlNewChild(node, NULL, "car", NULL);
-        cdr = xmlNewChild(node, NULL, "cdr", NULL);
+        car = xmlNewChild(node, NULL, (xmlChar *)"car", NULL);
+        cdr = xmlNewChild(node, NULL, (xmlChar *)"cdr", NULL);
 
         g_return_if_fail(gconf_value_get_car(value) != NULL);
         g_return_if_fail(gconf_value_get_cdr(value) != NULL);
@@ -611,7 +615,7 @@ find_schema_subnode_by_locale(xmlNodePtr node, const gchar* locale)
   while (iter != NULL)
     {
       if (iter->type == XML_ELEMENT_NODE &&
-          strcmp(iter->name, "local_schema") == 0)
+          strcmp((char *)iter->name, "local_schema") == 0)
         {
           char* this_locale = my_xmlGetProp(iter, "locale");
           
@@ -699,13 +703,14 @@ schema_subnode_extract_data(xmlNodePtr node, GConfSchema* sc)
     }
   else
     {
-      gconf_log(GCL_DEBUG, "found <local_schema> with no locale setting");
+      gconf_log(GCL_DEBUG, "found <%s> with no locale setting",
+                node->name ? node->name : (unsigned char*) "null");
     }
   
   if (node->xmlChildrenNode != NULL)
     {
       GConfValue* default_value = NULL;
-      gchar* ld_str = NULL;
+      xmlChar* ld_str = NULL;
       GSList* bad_nodes = NULL;
       xmlNodePtr iter = node->xmlChildrenNode;
 
@@ -714,7 +719,7 @@ schema_subnode_extract_data(xmlNodePtr node, GConfSchema* sc)
           if (iter->type == XML_ELEMENT_NODE)
             {
               if (default_value == NULL &&
-                  strcmp(iter->name, "default") == 0)
+                  strcmp((char *)iter->name, "default") == 0)
                 {
                   default_value = node_extract_value(iter, NULL, &error);
 
@@ -731,7 +736,7 @@ schema_subnode_extract_data(xmlNodePtr node, GConfSchema* sc)
                     }
                 }
               else if (ld_str == NULL &&
-                       strcmp(iter->name, "longdesc") == 0)
+                       strcmp((char *)iter->name, "longdesc") == 0)
                 {
                   ld_str = xmlNodeGetContent(iter);
                 }
@@ -768,7 +773,7 @@ schema_subnode_extract_data(xmlNodePtr node, GConfSchema* sc)
 
       if (ld_str)
         {
-          gconf_schema_set_long_desc(sc, ld_str);
+          gconf_schema_set_long_desc(sc, (char *)ld_str);
           xmlFree(ld_str);
         }
     }
@@ -833,16 +838,18 @@ schema_node_extract_value(xmlNodePtr node, const gchar** locales)
       type = gconf_value_type_from_string(cdr_type_str);
       gconf_schema_set_cdr_type(sc, type);
       xmlFree(cdr_type_str);
-    }
-
-  if (locales != NULL)
+    }  
+  
+  if (locales != NULL && locales[0])
     {
       /* count the number of possible locales */
-      i = 0;
-      while (locales[i])
-        ++i;
+      int n_locales;
       
-      localized_nodes = g_new0(xmlNodePtr, i+2);
+      n_locales = 0;
+      while (locales[n_locales])
+        ++n_locales;
+      
+      localized_nodes = g_new0(xmlNodePtr, n_locales);
       
       /* Find the node for each possible locale */
       iter = node->xmlChildrenNode;
@@ -850,7 +857,7 @@ schema_node_extract_value(xmlNodePtr node, const gchar** locales)
       while (iter != NULL)
         {
           if (iter->type == XML_ELEMENT_NODE &&
-              strcmp(iter->name, "local_schema") == 0)
+              strcmp((char *)iter->name, "local_schema") == 0)
             {
               char* locale_name;
               
@@ -885,7 +892,7 @@ schema_node_extract_value(xmlNodePtr node, const gchar** locales)
       
       i = 0;
       best = localized_nodes[i];
-      while (best == NULL && localized_nodes[i])
+      while (best == NULL && i < n_locales)
         {
           best = localized_nodes[i];
           ++i;
@@ -901,11 +908,15 @@ schema_node_extract_value(xmlNodePtr node, const gchar** locales)
     best = find_schema_subnode_by_locale (node, NULL);
 
   if (best == NULL)
-    best = node->xmlChildrenNode;
+    {
+      best = node->xmlChildrenNode;
+      while (best && best->type != XML_ELEMENT_NODE)
+        best = best->next;
+    }
   
   /* Extract info from the best locale node */
   if (best != NULL)
-    schema_subnode_extract_data(best, sc);  
+    schema_subnode_extract_data(best, sc); 
   
   /* Create a GConfValue with this schema and return it */
   value = gconf_value_new(GCONF_VALUE_SCHEMA);
@@ -993,11 +1004,11 @@ node_extract_value(xmlNodePtr node, const gchar** locales, GError** err)
               {
                 GConfValue* v = NULL;
 
-                if (strcmp(iter->name, "stringvalue") == 0)
+                if (strcmp((char *)iter->name, "stringvalue") == 0)
                   {
                     gchar* s;
 
-                    s = xmlNodeGetContent(iter);
+                    s = (gchar *)xmlNodeGetContent(iter);
 
                     v = gconf_value_new(GCONF_VALUE_STRING);
 
@@ -1061,7 +1072,7 @@ node_extract_value(xmlNodePtr node, const gchar** locales, GError** err)
             if (iter->type == XML_ELEMENT_NODE)
               {
                 GConfValue* v = NULL;
-                if (strcmp(iter->name, "li") == 0)
+                if (strcmp((char*)iter->name, "li") == 0)
                   {
                     
                     v = node_extract_value(iter, locales, err);
@@ -1122,7 +1133,7 @@ node_extract_value(xmlNodePtr node, const gchar** locales, GError** err)
           {
             if (iter->type == XML_ELEMENT_NODE)
               {
-                if (car == NULL && strcmp(iter->name, "car") == 0)
+                if (car == NULL && strcmp((char *)iter->name, "car") == 0)
                   {
                     car = node_extract_value(iter, locales, err);
                     if (car == NULL)
@@ -1144,7 +1155,7 @@ node_extract_value(xmlNodePtr node, const gchar** locales, GError** err)
                         car = NULL;
                       }
                   }
-                else if (cdr == NULL && strcmp(iter->name, "cdr") == 0)
+                else if (cdr == NULL && strcmp((char *)iter->name, "cdr") == 0)
                   {
                     cdr = node_extract_value(iter, locales, err);
                     if (cdr == NULL)
@@ -1234,7 +1245,7 @@ my_xmlSetProp(xmlNodePtr node,
 {
   xmlAttrPtr prop;
 
-  prop = xmlSetProp(node, name, str);
+  prop = xmlSetProp(node, (xmlChar *)name, (xmlChar *)str);
 
   if (str == NULL || *str == '\0')
     {
@@ -1267,9 +1278,9 @@ char*
 my_xmlGetProp(xmlNodePtr node,
               const gchar* name)
 {
-  char* prop;
+  xmlChar* prop;
 
-  prop = xmlGetProp(node, name);
+  prop = xmlGetProp(node, (xmlChar *)name);
 
   if (prop && *prop == '\0')
     {
@@ -1280,3 +1291,12 @@ my_xmlGetProp(xmlNodePtr node,
     return prop;
 }
 
+void
+xml_test_entry (void)
+{
+#ifndef GCONF_DISABLE_TESTS
+  
+
+
+#endif
+}
