@@ -18,11 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 
 #include "gconf-client.h"
-#include <gconf/gconf-internals.h>
+#include "gconf/gconf-internals.h"
 
 #include "gconfmarshal.h"
 #include "gconfmarshal.c"
@@ -644,6 +645,18 @@ foreach_add_notifies(gpointer key, gpointer value, gpointer user_data)
     }
 }
 
+static gboolean
+clear_dir_cache_foreach (char* key, GConfEntry* entry, char *dir)
+{
+  if (gconf_key_is_below (dir, key))
+    {
+      gconf_entry_free (entry);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+
 static void
 gconf_client_real_remove_dir    (GConfClient* client,
                                  Dir* d,
@@ -667,6 +680,10 @@ gconf_client_real_remove_dir    (GConfClient* client,
       d->notify_id = 0;
     }
   
+  g_hash_table_foreach_remove (client->cache_hash,
+                               (GHRFunc)clear_dir_cache_foreach,
+                               d->name);
+
   dir_destroy(d);
 
   ad.client = client;
@@ -760,6 +777,7 @@ gconf_client_notify (GConfClient* client, const char* key)
   if (entry != NULL)
     {
       notify_one_entry (client, entry);
+      gconf_entry_unref (entry);
     }
 }
 
@@ -1396,7 +1414,7 @@ gconf_client_get_float (GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, 0.0);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1431,7 +1449,7 @@ gconf_client_get_int   (GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, 0);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1465,7 +1483,7 @@ gconf_client_get_string(GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1501,7 +1519,7 @@ gconf_client_get_bool  (GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1535,7 +1553,7 @@ gconf_client_get_schema  (GConfClient* client,
 
   g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1569,7 +1587,7 @@ gconf_client_get_list    (GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, NULL);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
@@ -1608,7 +1626,7 @@ gconf_client_get_pair    (GConfClient* client, const gchar* key,
 
   g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 
-  val = gconf_client_get (client, key, err);
+  val = gconf_client_get (client, key, &error);
 
   if (val != NULL)
     {
