@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include "gconf.h"
 #include "gconf-internals.h"
 #include "gconf-sources.h"
@@ -42,9 +43,11 @@ main (int argc, char** argv)
 {
   GOptionContext *context;
   GError *error;
-  
+
+  g_thread_init (NULL);
+
   context = g_option_context_new (_("- Sanity checks for GConf"));
-  g_option_context_add_group (context, gtk_get_option_group (TRUE));
+  g_option_context_add_group (context, gtk_get_option_group (FALSE));
 
   error = NULL;
   g_option_context_parse (context, &argc, &argv, &error);
@@ -117,9 +120,23 @@ check_file_locking (void)
       GError *err;
 
       err = NULL;
-      fd = g_file_open_tmp ("gconf-test-locking-file-XXXXXX",
-                            &testfile,
-                            &err);
+      if (g_getenv ("GCONF_TMPDIR")) {
+	testfile = g_build_filename(g_getenv ("GCONF_TMPDIR"), "gconf-test-locking-file-XXXXXX", NULL);
+	fd = g_mkstemp (testfile);
+	if (fd == -1)
+	    {
+	      g_set_error (&err,
+        	           G_FILE_ERROR,
+                	   g_file_error_from_errno (errno),
+	                   "Failed to create file '%s': %s",
+			   testfile, g_strerror (errno));
+	    }
+    	}
+      else {
+	      fd = g_file_open_tmp ("gconf-test-locking-file-XXXXXX",
+        	                    &testfile,
+                	            &err);
+      }
 
       if (err != NULL)
         {
@@ -379,8 +396,6 @@ offer_delete_locks (void)
 
       g_slist_free (addresses);
       
-      gconf_daemon_blow_away_locks ();
-
       return TRUE;
     }
 
